package
{
	import flash.geom.Point;
	import net.flashpunk.Entity;
	import net.flashpunk.graphics.TiledImage;
	import net.flashpunk.masks.Grid;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.FP;
	import flash.utils.ByteArray;
	import flash.display.BitmapData;
	
	public class DynamicTilemap extends Entity
	{
		public var collision:Grid = null;
		public var map:Vector.<uint>;
		public var w:uint, h:uint,tw:uint,th:uint;
		public var tiles:Spritemap;
		
		public function DynamicTilemap(w:uint, h:uint)
		{
			// setting up the dimensions
			this.w = w;
			this.h = h;
			
			// creating a static sized tilemap
			map = new Vector.<uint>(w * h, true);
			
			// entity type for collisions
			type = "level";
			
		}
		
		// loading the level
		// you can extend this to create levels procedurally
		public function load(img:Class, map:Class,tile_w:uint,tile_h:uint):void
		{
			// setting up tile dimensions
			tw = tile_w;
			th = tile_h;
			
			// tile graphics
			tiles = new Spritemap(img, tw, th);
			
			// collision grid
			collision = new Grid(w * tw, h * th, tw, th, 0, 0);
			mask = collision;
			
			// reading the binary map file
			var b:ByteArray = new map();
			var maxTile:uint = 0;
			
			//iterating through the data and creating the level
			for (var yy:uint = 0; yy < h; yy ++)
			{
				for (var xx:uint = 0; xx < w; xx ++)
				{
					// read tile data
					var tile:uint = b.readUnsignedByte();
					
					
					// 0 tile is the no-tile thing
					// default tile Studio blank tile is 0xff (255)
					if (tile == 0xff) tile = 0;
					
					// here, we check if the tile is actually a tile
					// e.g. tile 0xc0 (192) represents
					// player starting position
					// add more cases to create enemies, coins or whatever
					if (tile == 0xc0)
					{
						// creating the player
						// not that the coords are multiplied 
						// by tile size
						FP.world.add(new Player(xx * tw, yy * th));
						tile = 0;
						
					} else if (tile == 0xc1)
					{
						// another case
						
					} else {
						// if not found anywhere
						set_at(xx, yy, tile);
						
						//use that for something
						if (tile > maxTile) maxTile = tile;
					}
					
				}
			}
			
			
			
		}
		
		public function get_at(x:uint, y:uint):uint
		{
			// if overboard, return
			if (x < 0 || y < 0 || x > w - 1 || y > h - 1) return 0;
			// else, return tile
			return map[x+y*w]
		}
		
		public function set_at(x:uint, y:uint,value:uint):void
		{
			// if overboard, return
			if (x < 0 || y < 0 || x > w - 1 || y > h - 1) return;
			
			// else set tile image
			map[x + y * w] = value;
			
			// and collision value
			// set up a condition here if you want non-solid tiles
			collision.setTile(x, y, value!=0);
		}
		
		override public function update():void
		{
			
		}
		
		override public function render():void
		{
			// this renders tiles that happen to be on the screen
			var sx:int, sy:int, ex:int, ey:int;
			sx = (FP.camera.x / tw) - 1;
			sy = (FP.camera.y / th) - 1;
			ex = sx + (FP.width / tw) + 2;
			ey = sy + (FP.height / th) + 3;
			if (sx < 0) sx = 0;
			if (sy < 0) sy = 0;
			if (ex > w-1) ex = w-1;
			if (ey > h-1) ey = h-1;
			if (sx > w || sy > h || ex < 0 || ey < 0) return;
			for (var xx:uint = sx; xx < ex; xx++)
			{
				for (var yy:uint = sy; yy < ey; yy++)
				{
					var tile:uint = get_at(xx, yy);
					if (!tile) continue;
					var point:Point = new Point(xx * tw + x, yy * th + y);
					tiles.frame = tile;
					tiles.render(FP.buffer, point, FP.camera);
				}
			}
		}
	}
}